/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/std/memory.hpp>

#include "context.hpp"
#include "module.hpp"

namespace nostalgia::foundation {

template<typename Ctx = foundation::Context>
ox::UPtr<Ctx> init(ox::UPtr<ox::FileSystem> &&fs, ox::CRStringView appName) noexcept {
	auto ctx = ox::make_unique<Ctx>();
	ctx->rom = std::move(fs);
	ctx->appName = appName;
	auto mods = modules();
	if (mods) {
		for (auto &mod : *mods) {
			// register type converters
			for (auto c : mod->converters()) {
				ctx->converters.emplace_back(c);
			}
			// register pack transforms
			for (auto c : mod->packTransforms()) {
				ctx->packTransforms.emplace_back(c);
			}
		}
	}
	return ctx;
}

}
