/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <ox/claw/read.hpp>

#include "typeconv.hpp"

namespace nostalgia::foundation {

#ifndef OX_BARE_METAL

[[nodiscard]]
static auto findConverter(foundation::Context *ctx,
                          ox::CRStringView srcTypeName, int srcTypeVersion,
                          ox::CRStringView dstTypeName, int dstTypeVersion) noexcept -> ox::Result<BaseConverter*> {
	for (auto &c : ctx->converters) {
		if (c->matches(srcTypeName, srcTypeVersion, dstTypeName, dstTypeVersion)) {
			return c;
		}
	}
	return OxError(1, "Could not find converter");
};

static ox::Result<ox::UniquePtr<Wrap>> convert(foundation::Context *ctx, const ox::Buffer &srcBuffer,
                                               ox::CRStringView srcTypeName, int srcTypeVersion,
                                               ox::CRStringView dstTypeName, int dstTypeVersion) noexcept {
	// look for direct converter
	auto [c, err] = findConverter(ctx, srcTypeName, srcTypeVersion, dstTypeName, dstTypeVersion);
	if (!err) {
		return c->convertBuffToPtr(nullptr, srcBuffer);
	}
	// try to chain multiple converters
	for (const auto &subConverter : ctx->converters) {
		if (!subConverter->dstMatches(dstTypeName, dstTypeVersion)) {
			continue;
		}
		const auto [intermediate, chainErr] =
			convert(ctx, srcBuffer, srcTypeName, srcTypeVersion,
			        subConverter->srcTypeName(), subConverter->srcTypeVersion());
		if (!chainErr) {
			return subConverter->convertPtrToPtr(ctx, intermediate.get());
		}
	}
	return OxError(1, "Could not convert between types");
}

ox::Result<ox::UniquePtr<Wrap>> convert(foundation::Context *ctx, const ox::Buffer &srcBuffer, ox::CRStringView dstTypeName, int dstTypeVersion) noexcept {
	oxRequire(hdr, ox::readClawHeader(srcBuffer));
	return convert(ctx, srcBuffer, hdr.typeName, hdr.typeVersion, dstTypeName, dstTypeVersion);
}

#endif

}
