/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/fs/fs.hpp>
#include <ox/std/error.hpp>
#include <ox/std/types.hpp>
#include <ox/std/vector.hpp>

namespace nostalgia::scene {

struct TileDoc {

	constexpr static auto TypeName = "net.drinkingtea.nostalgia.scene.TileDoc";
	constexpr static auto TypeVersion = 1;
	constexpr static auto Preloadable = true;

	ox::String sheetIdx;
	uint8_t type = 0;

};

oxModelBegin(TileDoc)
	oxModelFieldRename(sheet_idx, sheetIdx);
	oxModelField(type);
oxModelEnd()

struct SceneDoc {

	using TileMapRow = ox::Vector<TileDoc>;
	using TileMapLayer = ox::Vector<TileMapRow>;
	using TileMap = ox::Vector<TileMapLayer>;

	constexpr static auto TypeName = "net.drinkingtea.nostalgia.scene.SceneDoc";
	constexpr static auto TypeVersion = 1;
	constexpr static auto Preloadable = true;

	ox::FileAddress tilesheet;
	ox::FileAddress palette;
	TileMap tiles;

};

oxModelBegin(SceneDoc)
	oxModelField(tilesheet)
	oxModelField(palette)
	oxModelField(tiles)
oxModelEnd()

struct Scene {

	constexpr static auto TypeName = "net.drinkingtea.nostalgia.scene.Scene";
	constexpr static auto TypeVersion = 1;
	constexpr static auto Preloadable = true;

	struct Tile {
		uint16_t &tileMapIdx;
		uint8_t &tileType;
		constexpr Tile(uint16_t &pTileMapIdx, uint8_t &pTileType) noexcept:
			tileMapIdx(pTileMapIdx),
			tileType(pTileType) {
		}
	};
	struct Layer {
		uint16_t &columns;
		uint16_t &rows;
		uint16_t *tileMapIdx = nullptr;
		uint8_t *tileType = nullptr;
		constexpr Layer(uint16_t &pColumns,
		                uint16_t &pRows,
		                uint16_t *pTileMapIdx,
		                uint8_t *pTileType) noexcept:
			columns(pColumns),
			rows(pRows),
			tileMapIdx(pTileMapIdx),
			tileType(pTileType) {
		}
		[[nodiscard]]
		constexpr Tile tile(std::size_t i) const noexcept {
			return {tileMapIdx[i], tileType[i]};
		}
	};

	uint16_t layers = 0;
	ox::Vector<uint16_t> columns;
	ox::Vector<uint16_t> rows;
	ox::Vector<ox::Vector<uint16_t>> tileMapIdx;
	ox::Vector<ox::Vector<uint8_t>>  tileType;

	[[nodiscard]]
	constexpr Layer layer(std::size_t i) noexcept {
		return {columns[i], rows[i], tileMapIdx[i].data(), tileType[i].data()};
	}

};

oxModelBegin(Scene)
	oxModelField(layers)
	oxModelField(columns)
	oxModelField(rows)
	oxModelFieldRename(tile_map_idx, tileMapIdx)
	oxModelFieldRename(tile_type, tileType)
oxModelEnd()

}
